using AutoScan;
using AutoScan.Interfaces;
using CasaBotApp.TelegramBot;
using ControlServer;
using Microsoft.Extensions.Logging;
using System.Diagnostics;
using Telegram.Bots.Types;
using BotCommand = CasaBotApp.TelegramBot.BotCommand;

namespace CasaBotApp.Controllers;

public class BotController : IController
{
    private readonly BotHandler _botHandler;
    private readonly ILogger<BotController> _logger;
    private readonly AutoScanApp _autoScanApp;
    private readonly ISnapshoter _snapshoter;
    private readonly IControlServer _controlServer;


    public BotController(BotHandler botHandler, ILogger<BotController> logger, AutoScanApp autoScanApp, ISnapshoter snapshoter, IControlServer controlServer)
    {
        _botHandler = botHandler;
        _logger = logger;
        _autoScanApp = autoScanApp;
        _snapshoter = snapshoter;
        _controlServer = controlServer;
    }

    public void Register()
    {
        _logger.LogInformation("Registering bot commands");
        var methods = GetType().GetMethods()
            .Where(m => m.GetCustomAttributes(typeof(BotCommandAttribute), false).Length > 0)
            .ToArray();
        foreach (var method in methods)
        {
            try
            {
                var attribute = (BotCommandAttribute)method.GetCustomAttributes(typeof(BotCommandAttribute), false)[0];
                var command = new BotCommand
                {
                    Command = attribute.Command,
                    Description = attribute.Description,
                    Action = method.CreateDelegate<Func<TextMessage, BotCommand, Task>>(this),
                };
                _botHandler.RegisterCommand(command);
            
                _logger.LogInformation("Registered command: {Command} - {Description}", command.Command, command.Description);
            }
            catch (Exception e)
            {
                _logger.LogError(e, "Error registering command {Command}", method.Name);
            }
            
        }
        
        HandleReply();
    }

    [BotCommand("/soyandre", "Soy Andre")]
    public async Task HiAndre(TextMessage msg, BotCommand ctx)
    {
        _logger.LogInformation("Andre stoped by");
        await ctx.Responder(msg, "Hola vida, te amo mucho ❤️");
    }

    [BotCommand("/startScan", "Start a scan of last night images")]
    public async Task StartScan(TextMessage msg, BotCommand ctx)
    {
        await ctx.Responder(msg, "Starting scan 🔍📼");
        await _autoScanApp.StartNewScan();
    }

    [BotCommand("/lastscan", "Send the images from the last scan")]
    public async Task LastScan(TextMessage msg, BotCommand ctx)
    {
        var images = _autoScanApp.GetLastScanPictures();
        if (images.Length == 0)
        {
            await ctx.Responder(msg, "No images found");
            return;
        }
        await _botHandler.SendPhotos(msg.Chat.Id, images);
        
    }

    [BotCommand("/now", "Get the current snapshot")]
    public async Task CurrentSnapshot(TextMessage msg, BotCommand ctx)
    {
        var stopwatch = Stopwatch.StartNew();
        stopwatch.Start();
        var outputPath = await _snapshoter.TakeSnapshot();
        stopwatch.Stop();
        if (string.IsNullOrEmpty(outputPath))
        {
            await ctx.Responder(msg, "Error taking snapshot, try later");
            return;
        }
        _ = _botHandler.SendPhoto(msg.Chat.Id, outputPath, $"Snapshot: {DateTime.Now:g} ({stopwatch.ElapsedMilliseconds} ms)");
    }

    [BotCommand("/disarm", "Disarm the Door Sensor")]
    public async Task Disarm(TextMessage msg, BotCommand ctx)
    {
        await ctx.Responder(msg, "Disarming the door sensor");
        _controlServer.RequestDisarm();
    }

    private void HandleReply()
    {
        _botHandler.OnReply = async msg =>
        {
            var originalMsg = msg.ReplyToMessage;

            // Check if the original message is a photo and has a caption
            if (originalMsg is not PhotoMessage photoMessage || photoMessage.Caption is null)
                return;
            
            var videoPath = _autoScanApp.GetVideoPath(photoMessage.Caption);
            if (string.IsNullOrEmpty(videoPath))
            {
                await _botHandler.SendText(msg.Chat.Id, "No video found for this image");
                return;
            }
            
            await _botHandler.SendVideo(msg.Chat.Id, videoPath);
        };
    }
    
    [AttributeUsage(AttributeTargets.Method)]
    private class BotCommandAttribute : Attribute
    {
        public string Command { get; }
        public string Description { get; }

        public BotCommandAttribute(string command, string description)
        {
            Command = command;
            Description = description;
        }
    }
}